package com.community.huiju.controller;

import com.alibaba.fastjson.JSONObject;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.common.wxmp.WxMPUtil;
import com.community.huiju.common.wxpay.WXPayConstants;
import com.community.huiju.common.wxpay.WXPayUtil;
import com.community.huiju.feign.IFuShiServer;
import com.community.huiju.feign.entity.ApiThirdPartyMonthCardPay;
import com.community.huiju.service.WxMPUserServiceI;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.Map;

/**
 * @author FXF
 * @date 2019-07-08
 */
@RestController
@RefreshScope
@RequestMapping("/")
@Api(value = "微信公众号API", description = "微信公众号API")
public class WxMPUserController {
	
	private Logger log = LoggerFactory.getLogger(WxMPUserController.class);
	
	@Value("${wx.mp.configs.appId}")
	private String appId;
	
	@Autowired
	private WxMPUserServiceI wxMPUserService;
	
	@Autowired
	private IFuShiServer fuShiServer;

	@RequestMapping(value = "/wxmp/info", method = RequestMethod.GET)
	@ApiOperation(value = "获取微信公众号相关信息", notes = "获取微信公众号相关信息")
	@ApiImplicitParams({})
	public ResponseBean getWxMPinfo(){
		ResponseBean responseBean = new ResponseBean();
		Map<String,Object> map = Maps.newHashMap();
		map.put("appId",appId);
		responseBean.addSuccess(map);
		return responseBean;
	}
	
	@RequestMapping(value = "/{communityId}/wxmp", method = RequestMethod.GET)
	@ApiOperation(value = "根据CODE获取OPENID并判断是否第一次来,并返回数据", notes = "根据CODE获取OPENID并判断是否第一次来,并返回数据")
	@ApiImplicitParams({@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "code", value = "微信code")})
	public ResponseBean getOpenidByCode(@PathVariable Integer communityId, @RequestParam String code){
		ResponseBean responseBean = wxMPUserService.getTemporaryData(code,appId,communityId);
		return responseBean;
	}
	
	
	@RequestMapping(value = "/{communityId}/wxmp/records", method = RequestMethod.GET)
	@ApiOperation(value = "根据openid获取缴费记录", notes = "根据openid获取缴费记录")
	@ApiImplicitParams({@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "openid", value = "微信openid")})
	public ResponseBean getRecordsByOpenid(@PathVariable Integer communityId, @RequestParam String openid){
		ResponseBean responseBean = wxMPUserService.getRecordsByOpenid(openid,communityId);
		return responseBean;
	}
	
	@RequestMapping(value = "/{communityId}/wxmp/car", method = RequestMethod.GET)
	@ApiOperation(value = "根据车牌获取数据", notes = "根据车牌获取数据")
	@ApiImplicitParams({@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "carNo", value = "车牌号")})
	public ResponseBean getParkInfoByCarNo(@PathVariable Integer communityId, @RequestParam String carNo){
		ResponseBean responseBean = new ResponseBean();
		String parkInfoByCarNo = fuShiServer.getParkInfoByCarNo(carNo);
		JSONObject carNoJson = JSONObject.parseObject(parkInfoByCarNo);
		//车辆不在停车场
		if (carNoJson.getInteger("MessageCode") == 44036 ){
			responseBean.addError("未查询到入场信息，请确认车牌号或联系保安出场");
			return responseBean;
		}
		
		if (carNoJson.getInteger("MessageCode") == 200 ){
			if (carNoJson.getInteger("DataType") != 3){
				responseBean.addSuccess("储值车不支持在线缴费，月卡车请使用银城慧家APP延期续费");
				return responseBean;
			}
			//返回查询的金额
			JSONObject jsonParam = carNoJson.getJSONObject("JsonParam");
			responseBean.addSuccess(jsonParam.get("ActualAmount"));
		}
		return responseBean;
	}
	
	@ApiOperation(value = "临时车统一下单", notes = "临时车统一下单")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "carNo", value = "车牌号"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "openid", value = "微信openid")
	})
	@RequestMapping(value = "/{communityId}/wxmp/order",method = RequestMethod.GET)
	public ResponseBean wxTmpOrder(@PathVariable Integer communityId,
	                               @RequestParam("carNo") String carNo,
	                               @RequestParam("openid") String openid){
		ResponseBean responseBean = new ResponseBean();
		Map<String, String> resp = Maps.newHashMap();
		try {
			String parkInfoByCarNo = fuShiServer.getParkInfoByCarNo(carNo);
			JSONObject carNoJson = JSONObject.parseObject(parkInfoByCarNo);
			String price = String.valueOf(Double.valueOf(((BigDecimal) carNoJson.getJSONObject("JsonParam").get("ActualAmount")).toString()) * 100);
			price = price.substring(0, price.lastIndexOf("."));
			resp = wxMPUserService.wxTmpOrder(carNo,price,communityId,openid);
		} catch (Exception e) {
			e.printStackTrace();
			responseBean.addError(e.getMessage());
			return responseBean;
		}
		responseBean.addSuccess(resp);
		return responseBean;
	}
	
	@ApiOperation(value = "手机订单支付完成后回调临时车缴费", notes = "手机订单支付完成后回调临时车缴费")
	@RequestMapping(value = "/wxTmpNotify",method = RequestMethod.POST)
	public String wxBillNotify(HttpServletRequest request, HttpServletResponse response){
		String failResult = "<xml>" + "<return_code><![CDATA[FAIL]]></return_code>" + "<return_msg><![CDATA[失败]]></return_msg>" + "</xml> ";
		String successResult = "<xml>" + "<return_code><![CDATA[SUCCESS]]></return_code>" + "<return_msg><![CDATA[OK]]></return_msg>" + "</xml> ";
		InputStream inStream;
		try {
			
			inStream = request.getInputStream();
			ByteArrayOutputStream outSteam = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int len = 0;
			while ((len = inStream.read(buffer)) != -1) {
				outSteam.write(buffer, 0, len);
			}
			
			// 获取微信调用我们notify_url的返回信息
			String result = new String(outSteam.toByteArray(), "utf-8");
			WXPayUtil.getLogger().info("wxnotify:微信支付----result----=" + result);
			
			// 关闭流
			outSteam.close();
			inStream.close();
			
			// xml转换为map
			Map<String, String> resultMap = WXPayUtil.xmlToMap(result);
			if (WXPayConstants.SUCCESS.equalsIgnoreCase(resultMap.get("result_code"))) {
				//推送富士系统
				// TODO: 2019/7/8
				//处理业务逻辑 修改订单状态等
				String state = wxMPUserService.updateTmpOrder(resultMap.get("out_trade_no"));
				if (state.equals("success")){
					log.info("微信支付成功。。");
					return successResult;
				}
			}else{
				return failResult;
			}
			
		} catch (Exception e) {
			log.error("手机支付回调通知失败",e);
			return failResult;
			
		}
		
		return failResult;
	}
}
