package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.community.commom.mode.ResponseBean;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.feign.IHKServer;
import com.community.huiju.feign.entity.AddFace;
import com.community.huiju.feign.entity.PushPerson;
import com.community.huiju.feign.entity.UpdateFace;
import com.community.huiju.model.*;
import com.community.huiju.service.IEquipmentService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

@Slf4j
@Service
public class EquipmentServiceImpl implements IEquipmentService {

    @Autowired
    private IHKServer ihkServer;

    private ResponseBean getResponseBean(String result){
        JSONObject jsonObject = JSONObject.parseObject(result);
        ResponseBean responseBean = jsonObject.toJavaObject(ResponseBean.class);
        if ("1".equals(responseBean.getCode())) {
            throw new WisdomException(responseBean.getMessage());
        }

        responseBean.addSuccess(jsonObject.getJSONObject("data").getInnerMap());
        return responseBean;
    }

    @Override
    public String pushPerson(TpEquipmentTree tpEquipmentTree, TpUnitHkSetting unitHkSetting, TaUser user, TaUserHk taUserHk, Integer expirationTime) {

        // 人员信息
        PushPerson.PersonBean personBean = new PushPerson.PersonBean();
        personBean.setAppkey(tpEquipmentTree.getAppkey());
        personBean.setSecret(tpEquipmentTree.getSecret());
        personBean.setOpUserUuid(tpEquipmentTree.getOpUserUuid());
        personBean.setOpenapi_ip_port_http(tpEquipmentTree.getHttpServer());
        personBean.setDeptUuid(unitHkSetting.getDepartmentId());
        personBean.setPersonName(user.getUserName());
        personBean.setPersonNo(taUserHk.getHkPersonNo());
        personBean.setRemark(user.getLoginName());
        personBean.setCard(Long.valueOf(taUserHk.getHkCardNo()));
        personBean.setExpirationTime(expirationTime);

        PushPerson pushPerson = new PushPerson();
        pushPerson.setType(tpEquipmentTree.getFactory());
        pushPerson.setPerson(personBean);

        String result = ihkServer.pushPerson(JSONObject.toJSONString(pushPerson));
        ResponseBean responseBean = getResponseBean(result);
        return JSONObject.toJSONString(responseBean.getData());
    }

    @Override
    public String addFace(MultipartFile faceImg, TpUnitHkSetting unitHkSetting, TpEquipmentTree tpEquipmentTree,
                          TaUserHk taUserHk, TaUserVerify userVerify, TaUser user, List<String> visualList,
                          String doorUuids, String deviceUuids) {


        AddFace addFace = new AddFace();
        AddFace.FaceBean faceBean = new AddFace.FaceBean();

        // 人脸图片
        AddFace.FaceBean.PersonBean person = new AddFace.FaceBean.PersonBean();
        person.setAppkey(tpEquipmentTree.getAppkey());
        person.setSecret(tpEquipmentTree.getSecret());
        person.setOpenapi_ip_port_http(tpEquipmentTree.getHttpServer());
        person.setOpUserUuid(tpEquipmentTree.getOpUserUuid());
        person.setPersonId(taUserHk.getHkUserId());
        person.setPersonNo(Long.valueOf(taUserHk.getHkCardNo()));
        person.setPersonName(user.getUserName());
        person.setCardNo(taUserHk.getHkCardNo());

        faceBean.setPerson(person);


        if (CollectionUtils.isNotEmpty(visualList)) {
            // 门口机卡权限(可是对讲)
            AddFace.FaceBean.DoorPermissionsBean doorPermissionsBean = new AddFace.FaceBean.DoorPermissionsBean();
            doorPermissionsBean.setAppkey(tpEquipmentTree.getAppkey());
            doorPermissionsBean.setSecret(tpEquipmentTree.getSecret());
            doorPermissionsBean.setOpenapi_ip_port_http(tpEquipmentTree.getHttpServer());
            doorPermissionsBean.setOpUserUuid(tpEquipmentTree.getOpUserUuid());
            doorPermissionsBean.setAuthName(user.getUserName()+userVerify.getId());
            doorPermissionsBean.setPersonIds(Arrays.asList(taUserHk.getHkUserId()));
            doorPermissionsBean.setLongNums(visualList);

            // 门口机人脸权限(可视对讲)
            AddFace.FaceBean.DoorFaceBean doorFaceBean = new AddFace.FaceBean.DoorFaceBean();
            doorFaceBean.setAppkey(tpEquipmentTree.getAppkey());
            doorFaceBean.setSecret(tpEquipmentTree.getSecret());
            doorFaceBean.setOpenapi_ip_port_http(tpEquipmentTree.getHttpServer());
            doorFaceBean.setOpUserUuid(tpEquipmentTree.getOpUserUuid());
            doorFaceBean.setOpType(1);
            doorFaceBean.setLongNums(visualList);
            doorFaceBean.setPersonIds(Arrays.asList(taUserHk.getHkUserId()));

            faceBean.setDoorPermissions(doorPermissionsBean);
            faceBean.setDoorFace(doorFaceBean);
        }


        if (org.apache.commons.lang3.StringUtils.isNotBlank(doorUuids)) {
            // 门禁卡权限下发
            AddFace.FaceBean.EntranceGuardPermissionsBean entranceGuardPermissionsBean = new AddFace.FaceBean.EntranceGuardPermissionsBean();
            entranceGuardPermissionsBean.setAppkey(tpEquipmentTree.getAppkey());
            entranceGuardPermissionsBean.setSecret(tpEquipmentTree.getSecret());
            entranceGuardPermissionsBean.setOpenapi_ip_port_http(tpEquipmentTree.getHttpServer());
            entranceGuardPermissionsBean.setOpUserUuid(tpEquipmentTree.getOpUserUuid());
            entranceGuardPermissionsBean.setPersonIds(String.valueOf(taUserHk.getHkUserId()));
            entranceGuardPermissionsBean.setDoorUuids(doorUuids.substring(0, doorUuids.lastIndexOf(",")));

            faceBean.setEntranceGuardPermissions(entranceGuardPermissionsBean);
        }

        if (org.apache.commons.lang3.StringUtils.isNotBlank(deviceUuids)) {

            // 门禁人脸权限下发
            AddFace.FaceBean.EntranceGuardFaceBean entranceGuardFaceBean = new AddFace.FaceBean.EntranceGuardFaceBean();
            entranceGuardFaceBean.setAppkey(tpEquipmentTree.getAppkey());
            entranceGuardFaceBean.setSecret(tpEquipmentTree.getSecret());
            entranceGuardFaceBean.setOpenapi_ip_port_http(tpEquipmentTree.getHttpServer());
            entranceGuardFaceBean.setOpUserUuid(tpEquipmentTree.getOpUserUuid());
            entranceGuardFaceBean.setDeviceUuids(deviceUuids.substring(0, deviceUuids.lastIndexOf(",")));
            entranceGuardFaceBean.setType(1);
            entranceGuardFaceBean.setPersonIds(String.valueOf(taUserHk.getHkUserId()));

            faceBean.setEntranceGuardFace(entranceGuardFaceBean);
        }

        addFace.setType(tpEquipmentTree.getFactory());
        addFace.setFace(faceBean);
        // 添加人脸
        String result = ihkServer.addFace(faceImg, JSONObject.toJSONString(addFace));
        ResponseBean responseBean = getResponseBean(result);
        return JSONObject.toJSONString(responseBean.getData());
    }

    @Override
    public String updateFace(MultipartFile faceImg, TpUnitHkSetting unitHkSetting, TpEquipmentTree tpEquipmentTree, TaUserHk taUserHk, TaUserVerify userVerify, TaUser user, List<String> visualList, String deviceUuids) {

        UpdateFace addFace = new UpdateFace();
        UpdateFace.FaceBean faceBean = new UpdateFace.FaceBean();

        // 人脸图片
        UpdateFace.FaceBean.PersonBean person = new UpdateFace.FaceBean.PersonBean();
        person.setAppkey(tpEquipmentTree.getAppkey());
        person.setSecret(tpEquipmentTree.getSecret());
        person.setOpenapi_ip_port_http(tpEquipmentTree.getHttpServer());
        person.setOpUserUuid(tpEquipmentTree.getOpUserUuid());
        person.setPersonId(taUserHk.getHkUserId());
        person.setPersonNo(Long.valueOf(taUserHk.getHkCardNo()));
        person.setPersonName(user.getUserName());
        person.setCardNo(taUserHk.getHkCardNo());

        faceBean.setPerson(person);


        if (CollectionUtils.isNotEmpty(visualList)) {

            // 门口机人脸权限(可视对讲)
            UpdateFace.FaceBean.DoorFaceBean doorFaceBean = new UpdateFace.FaceBean.DoorFaceBean();
            doorFaceBean.setAppkey(tpEquipmentTree.getAppkey());
            doorFaceBean.setSecret(tpEquipmentTree.getSecret());
            doorFaceBean.setOpenapi_ip_port_http(tpEquipmentTree.getHttpServer());
            doorFaceBean.setOpUserUuid(tpEquipmentTree.getOpUserUuid());
            doorFaceBean.setOpType(1);
            doorFaceBean.setLongNums(visualList);
            doorFaceBean.setPersonIds(Arrays.asList(taUserHk.getHkUserId()));

            faceBean.setDoorFace(doorFaceBean);
        }

        if (org.apache.commons.lang3.StringUtils.isNotBlank(deviceUuids)) {

            // 门禁人脸权限下发
            UpdateFace.FaceBean.EntranceGuardFaceBean entranceGuardFaceBean = new UpdateFace.FaceBean.EntranceGuardFaceBean();
            entranceGuardFaceBean.setAppkey(tpEquipmentTree.getAppkey());
            entranceGuardFaceBean.setSecret(tpEquipmentTree.getSecret());
            entranceGuardFaceBean.setOpenapi_ip_port_http(tpEquipmentTree.getHttpServer());
            entranceGuardFaceBean.setOpUserUuid(tpEquipmentTree.getOpUserUuid());
            entranceGuardFaceBean.setDeviceUuids(deviceUuids.substring(0, deviceUuids.lastIndexOf(",")));
            entranceGuardFaceBean.setType(1);
            entranceGuardFaceBean.setPersonIds(String.valueOf(taUserHk.getHkUserId()));

            faceBean.setEntranceGuardFace(entranceGuardFaceBean);
        }

        addFace.setType(tpEquipmentTree.getFactory());
        addFace.setFace(faceBean);
        // 添加人脸
        String result = ihkServer.updateFace(faceImg, JSONObject.toJSONString(addFace));
        ResponseBean responseBean = getResponseBean(result);
        return JSONObject.toJSONString(responseBean.getData());
    }
}
