package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.community.commom.mode.ResponseBean;
import com.community.huiju.base.api.hk.HKOpenApi;
import com.community.huiju.factory.PlatformVendors;
import com.community.huiju.factory.SimplePlatformVendorsFactory;
import com.community.huiju.service.HkHardwareServiceI;
import com.community.huiju.vendor.constants.dao.IPersonOperation;
import com.community.huiju.vendor.constants.info.PlatformVendorsInfo;
import com.community.huiju.vendor.impl.hk.HKCombination;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 海康硬件接口
 * @author FXF
 * @date 2019-07-03
 */
@Service
@Slf4j
public class HkHardwareServiceImpl implements HkHardwareServiceI {

    @Override
    public ResponseBean downloadDoorPermissions(String jsonString) {
        Map<String, Object> innerMap = JSONObject.parseObject(jsonString).getInnerMap();
        String result = HKOpenApi.addOutDoorAuth(innerMap);
        ResponseBean responseBean = new ResponseBean();
        responseBean.addSuccess(JSONObject.parseObject(result).getInnerMap());
        return responseBean;
    }

    @Override
    public ResponseBean downloadDoorFace(String jsonString) {
        Map<String, Object> innerMap = JSONObject.parseObject(jsonString).getInnerMap();

        String secret = (String) innerMap.get("secret");
        String appkey = (String) innerMap.get("appkey");
        String openapi_ip_port_http = (String) innerMap.get("openapi_ip_port_http");
        String opUserUuid = (String) innerMap.get("opUserUuid");
        Integer opType = (Integer) innerMap.get("opType"); // 1 表示下发
        List<Integer> personIds = (List<Integer>) innerMap.get("personIds");
        List<String> longNums = (List<String>) innerMap.get("longNums");

        String result = HKOpenApi.visualIntercom(opUserUuid, opType, personIds, longNums, appkey, secret,openapi_ip_port_http);
        ResponseBean responseBean = new ResponseBean();
        responseBean.addSuccess(JSONObject.parseObject(result).getInnerMap());
        return responseBean;
    }

    @Override
    public ResponseBean syncControl(String jsonString) {
        JSONObject jsonObject = JSONObject.parseObject(jsonString);
        Map<String, Object> innerMap = jsonObject.getInnerMap();

        ResponseBean responseBean = new ResponseBean();
        String result = HKOpenApi.syncControl(innerMap);
        responseBean.addSuccess(JSONObject.parseObject(result).getInnerMap());
        return responseBean;
    }

    @Override
    public ResponseBean deleteDoorPermissions(String jsonString) {
        return null;
    }

    @Override
    public ResponseBean downloadEntranceGuardPermissions(String jsonString) {
        Map<String,Object> map = JSONObject.parseObject(jsonString, HashMap.class);
        // 添加门禁权限
        String result = HKOpenApi.addAuthoritiesByPersonIds(map);

        // 下发门禁权限，还需要这两个参数
        map.put("operateType",1);
        map.put("includeFinger",1);

        result = HKOpenApi.downloadAuthorityByPersonIds(map);
        ResponseBean responseBean = new ResponseBean();
        responseBean.addSuccess(JSONObject.parseObject(result).getInnerMap());
        return responseBean;
    }

    @Override
    public ResponseBean downloadEntranceGuardFace(String jsonString) {
        Map<String, Object> innerMap = JSONObject.parseObject(jsonString).getInnerMap();
        String result = HKOpenApi.downloadFaceAndFingerInfos(innerMap);
        ResponseBean responseBean = new ResponseBean();
        responseBean.addSuccess(JSONObject.parseObject(result).getInnerMap());
        return responseBean;
    }

    @Override
    public ResponseBean deleteEntranceGuardPermissions(String jsonString) {
        return null;
    }

    @Override
    public ResponseBean addFace(MultipartFile faceImg, String jsonString) {
        String result = updateFaceImage(faceImg, jsonString);
        ResponseBean responseBean = new ResponseBean();
        responseBean.addSuccess(JSONObject.parseObject(result).getInnerMap());
        return responseBean;
    }

    private String updateFaceImage(MultipartFile faceImg, String jsonString) {
        JSONObject jsonObject = JSONObject.parseObject(jsonString);
        Map<String, Object> innerMap = jsonObject.getJSONObject("person").getInnerMap();

        String secret = (String) innerMap.get("secret");
        String appkey = (String) innerMap.get("appkey");
        String openapi_ip_port_http = (String) innerMap.get("openapi_ip_port_http");
        String opUserUuid = (String) innerMap.get("opUserUuid");
        Integer personId = (Integer) innerMap.get("personId");
        Long personNo = (Long) innerMap.get("personNo");
        String personName = (String) innerMap.get("personName");
        String cardNo = (String) innerMap.get("cardNo");

        // 修改人脸
        String result = HKOpenApi.HKpersonGroupId(faceImg, personId, personNo, personName, cardNo, appkey, secret, openapi_ip_port_http, opUserUuid);

        // 下发门禁卡权限
        JSONObject entranceGuardPermissions = jsonObject.getJSONObject("entranceGuardPermissions");
        if (null != entranceGuardPermissions) {
            downloadEntranceGuardPermissions(entranceGuardPermissions.toJSONString());
        }

        // 下发门禁人脸权限
        JSONObject entranceGuardFace = jsonObject.getJSONObject("entranceGuardFace");
        if (null != entranceGuardFace) {
            downloadEntranceGuardFace(entranceGuardFace.toJSONString());
        }

        // 下发门口机(可视对讲)卡权限
        JSONObject doorPermissions = jsonObject.getJSONObject("doorPermissions");
        if (null != doorPermissions) {
            downloadDoorPermissions(doorPermissions.toJSONString());
        }

        // 下发门口机(可视对讲)人脸权限
        JSONObject doorFace = jsonObject.getJSONObject("doorFace");
        if (null != doorFace) {
            downloadDoorFace(doorFace.toJSONString());
        }

        return result;
    }

    @Override
    public ResponseBean updateFace(MultipartFile faceImg, String jsonString) {
        String result = updateFaceImage(faceImg, jsonString);
        ResponseBean responseBean = new ResponseBean();
        responseBean.addSuccess(JSONObject.parseObject(result).getInnerMap());
        return responseBean;
    }

    @Override
    public ResponseBean deleteFace(String jsonString) {
        return null;
    }

    @Override
    public ResponseBean addPerson(String jsonString) {
        ResponseBean responseBean = new ResponseBean();

        JSONObject jsonObject = JSONObject.parseObject(jsonString);
        Map<String, Object> personMap = jsonObject.getInnerMap();
        String personResult = HKOpenApi.addUser(personMap);
        Map<String, Object> resultPersonMap = JSONObject.parseObject(personResult, HashMap.class);
        Map<String, Object> resultPersonDataMap = (Map<String, Object>) resultPersonMap.get("data");

        String openapi_ip_port_http = (String) personMap.get("openapi_ip_port_http");
        String appkey = (String) personMap.get("appkey");
        String secret = (String) personMap.get("secret");
        String opUserUuid = (String) personMap.get("opUserUuid");

        Long card = System.currentTimeMillis();

        // 添加卡片
        addCards(appkey, secret, openapi_ip_port_http, opUserUuid, card, card);

        // 人员开卡
        openCard(appkey, secret, openapi_ip_port_http, opUserUuid,(Integer) resultPersonDataMap.get("personId") ,card,0);

        Map<String, Object> map = new HashMap<>();
        map.put("person", resultPersonMap);
        map.put("card", card + "");

        responseBean.addSuccess(map);
        return responseBean;
    }

    @Override
    public ResponseBean updatePerson(String jsonString) {
        JSONObject jsonObject = JSONObject.parseObject(jsonString);
        Map<String, Object> personMap = jsonObject.getInnerMap();
        String result = HKOpenApi.updateUser(personMap);
        ResponseBean responseBean = new ResponseBean();
        responseBean.addSuccess(JSONObject.parseObject(result, HashMap.class));
        return responseBean;
    }

    @Override
    public ResponseBean deletePerson(String jsonString) {
        return null;
    }

    /**
     * 添加卡片
     * @return
     */
    public String addCards(String appkey, String secret, String openapiIpPortHttp , String opUserUuid, Long startCardNo, Long endCardNo) {

        ResponseBean response = new ResponseBean();

        // 开卡 卡号
        long cardNo = System.currentTimeMillis();

        // 添加卡片
        Map<String,Object> addCardsMap = new HashMap<>();
        addCardsMap.put("secret", secret);
        addCardsMap.put("appkey", appkey);
        addCardsMap.put("openapi_ip_port_http", openapiIpPortHttp);
        addCardsMap.put("startCardNo", cardNo);
        addCardsMap.put("endCardNo", cardNo);
        addCardsMap.put("opUserUuid", opUserUuid);
        String addCardsJSON = com.community.commom.hk.HKOpenApi.addCards(addCardsMap);
        return addCardsJSON;
    }


    /**
     * 卡片开卡
     *
     * @param personId 人员海康id
     * @param cardNo 卡号
     * @param expirationTime    有效期为多少? 单位为 年
     * @return
     */
    private String openCard(String appkey, String secret, String openapiIpPortHttp , String opUserUuid, Integer personId, Long cardNo, Integer expirationTime) {
        ResponseBean response = new ResponseBean();

        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        LocalDate today = LocalDate.now();
        int year = today.getYear();
        // 加上20, 表示加上20年,默认
        if (null == expirationTime || expirationTime <= 0) {
            year += 20;
        } else {
            year += expirationTime;
        }
        int month = today.getMonthValue();
        int day = today.getDayOfMonth();

        Long time = null;
        try {
            time = formatter.parse(year + "-" + month + "-" + day).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }

        // 开卡
        Map<String, Object> parOpenCard = Maps.newHashMap();
        parOpenCard.put("personId", personId);
        parOpenCard.put("cardNo", cardNo);
        parOpenCard.put("startTime", System.currentTimeMillis());
        parOpenCard.put("endTime", time);
        parOpenCard.put("secret", secret);
        parOpenCard.put("appkey", appkey);
        parOpenCard.put("openapi_ip_port_http", openapiIpPortHttp);
        parOpenCard.put("opUserUuid", opUserUuid);
        String result = HKOpenApi.openCard(parOpenCard);
        return result;
    }

}
